//===========================================================================
// @(#) $Name:  $
// @(#) $Id: DwmWWWGalleryDropUpload.cc,v 1.5 2015/05/02 08:35:41 dwm Exp $
//===========================================================================
//  Copyright (c) Daniel W. McRobb 2015
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//  1. Redistributions of source code must retain the above copyright
//     notice, this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright
//     notice, this list of conditions and the following disclaimer in the
//     documentation and/or other materials provided with the distribution.
//  3. The names of the authors and copyright holders may not be used to
//     endorse or promote products derived from this software without
//     specific prior written permission.
//
//  IN NO EVENT SHALL DANIEL W. MCROBB BE LIABLE TO ANY PARTY FOR
//  DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
//  INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE,
//  EVEN IF DANIEL W. MCROBB HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGE.
//
//  THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND
//  DANIEL W. MCROBB HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
//  UPDATES, ENHANCEMENTS, OR MODIFICATIONS. DANIEL W. MCROBB MAKES NO
//  REPRESENTATIONS AND EXTENDS NO WARRANTIES OF ANY KIND, EITHER
//  IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//  WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE,
//  OR THAT THE USE OF THIS SOFTWARE WILL NOT INFRINGE ANY PATENT,
//  TRADEMARK OR OTHER RIGHTS.
//===========================================================================

//---------------------------------------------------------------------------
//!  \file DwmWWWGalleryDropUpload.cc
//!  \brief DropUpload class implementation.
//---------------------------------------------------------------------------

#include <regex>
#include <sstream>

#include "DwmWWWGalleryDropUpload.hh"

using namespace std;

namespace Dwm {

  namespace WWW {

    namespace Gallery {

      //----------------------------------------------------------------------
      //!  
      //----------------------------------------------------------------------
      DropUpload::DropUpload(Wt::WContainerWidget *parent)
          : WLabel(parent), _uploadResource(this),
            _fileTooLarge(this), _dropUploadTypes(), _dragoverStyle()
      {
        addStyleClass("galleryDropUpload", true);
        setToolTip("Drop files here to upload");
        setJavaScriptMember("dragoverStyle", "'dropUploadDragover'");
        setJavaScriptMember("dropStyle", "'dropUploadDrop'");
        setJavaScriptMember("MimeTypes", "[]");
        CreateListeners();
      }

      //----------------------------------------------------------------------
      //!  
      //----------------------------------------------------------------------
      DropUpload::DropUpload(const Wt::WString & text,
                             Wt::WContainerWidget *parent)
          : Wt::WLabel(text, parent), _uploadResource(this),
            _fileTooLarge(this), _dropUploadTypes(), _dragoverStyle()
      {
        addStyleClass("galleryDropUpload", true);
        setToolTip("Drop files here to upload");
        setJavaScriptMember("dragoverStyle", "'dropUploadDragover'");
        setJavaScriptMember("dropStyle", "'dropUploadDrop'");
        setJavaScriptMember("MimeTypes", "[]");
        CreateListeners();
      }
      
      //----------------------------------------------------------------------
      //!  
      //----------------------------------------------------------------------
      DropUpload::DropUpload(Wt::WImage *image, Wt::WContainerWidget *parent)
          : Wt::WLabel(image, parent), _uploadResource(this),
            _fileTooLarge(this), _dropUploadTypes(), _dragoverStyle()
      {
        addStyleClass("galleryDropUpload", true);
        setToolTip("Drop files here to upload");
        setJavaScriptMember("dragoverStyle", "'dropUploadDragover'");
        setJavaScriptMember("dropStyle", "'dropUploadDrop'");
        setJavaScriptMember("MimeTypes", "[]");
        CreateListeners();
      }
      
      //----------------------------------------------------------------------
      //!  
      //----------------------------------------------------------------------
      bool DropUpload::getNextFile(Wt::Http::UploadedFile & uploadedFile)
      {
        return _uploadResource.getNextFile(uploadedFile);
      }

      //----------------------------------------------------------------------
      //!  
      //----------------------------------------------------------------------
      bool DropUpload::getNextURI(string & uri)
      {
        return _uploadResource.getNextURI(uri);
      }

      //----------------------------------------------------------------------
      //!  
      //----------------------------------------------------------------------
      void DropUpload::filesUploaded()
      {
        uploaded().emit();
        return;
      }

      //----------------------------------------------------------------------
      //!  
      //----------------------------------------------------------------------
      void DropUpload::fileTooLarge(int64_t tooLargeSize)
      {
        fileTooLarge().emit(tooLargeSize);
        return;
      }

      //----------------------------------------------------------------------
      //!  
      //----------------------------------------------------------------------
      void DropUpload::signalURIDropped()
      {
        uriDropped().emit();
        return;
      }
      
      //----------------------------------------------------------------------
      //!  
      //----------------------------------------------------------------------
      Wt::EventSignal<> & DropUpload::uploaded()
      {
        return *voidEventSignal("M_uploaded", true);
      }

      //----------------------------------------------------------------------
      //!  
      //----------------------------------------------------------------------
      Wt::EventSignal<> & DropUpload::uriDropped()
      {
        return *voidEventSignal("M_uriDropped", true);
      }
      
      //----------------------------------------------------------------------
      //!  
      //----------------------------------------------------------------------
      void DropUpload::CreateListeners()
      {
        CreateJavascriptUtilityFunctions();
        CreateDropListener();
        CreateDragoverListeners();
        return;
      }

      //----------------------------------------------------------------------
      //!  
      //----------------------------------------------------------------------
      void DropUpload::CreateJavascriptUtilityFunctions()
      {
        ostringstream  os;
        os << "function(e) {\n"
           << "  if (e.dataTransfer.types) {\n"
           << "    for (var i = 0; i < e.dataTransfer.types.length; ++i) {\n"
           << "      console.log('dragged type: ',e.dataTransfer.types[i]);\n"
           << "      if (e.dataTransfer.types[i] == 'Files') {\n"
           << "        return true;\n"
           << "      }\n"
           << "    }\n"
           << "  }\n"
           << "  return false;\n"
           << "}\n";
        setJavaScriptMember("EventContainsFiles", os.str());
        os.str("");

        os << "function(e) {\n"
           << "  var rc = false;\n"
           << "  if (e.dataTransfer.types) {\n"
           << "    for (var i = 0; i < e.dataTransfer.types.length; ++i) {\n"
           << "      console.log('dragged type: ',e.dataTransfer.types[i]);\n"
           << "      if (e.dataTransfer.types[i] == 'text/uri-list') {\n"
           << "        rc = true;\n"
           << "      }\n"
           << "    }\n"
           << "  }\n"
           << "  return rc;\n"
           << "}\n";
        setJavaScriptMember("EventContainsURIs", os.str());
        os.str("");
        
#if 0
        os << "function(f) {\n"
           << "  for (var i = 0; i < f.length; ++i) {\n"
           << "    for (var j = 0;"
           << "j < " << jsRef() << ".MimeTypes.length; ++j) {\n"
           << "      var patt = " << jsRef() << ".MimeTypes[j];\n"
           << "      var re = new RegExp(patt);\n"
           << "      var m = f[i].type.match(re);\n"
           << "      if (m != null) {\n"
           << "        return true;\n"
           << "      }\n"
           << "    }\n"
           << "  }\n"
           << "  return false;\n"
           << "}\n";
        setJavaScriptMember("FilesHaveMatchingMime", os.str());
        os.str("");
#endif
        
        os << "function(e) {"
           << "  var rc = [];\n"
           << "  if (" << jsRef() << ".EventContainsFiles(e)) {\n"
           << "    var f = e.dataTransfer.files;\n"
           << "    for (var i = 0;"
           << "         i < " << jsRef() << ".MimeTypes.length; ++i) {\n"
           << "      var  patt = " << jsRef() << ".MimeTypes[i];\n"
           << "      var  re = new RegExp(patt);\n"
           << "      var  numMatches = 0;\n"
           << "      for (var j = 0; j < f.length; ++j) {\n"
           << "        if (f[j].type.match(re)) {\n"
           << "          rc[numMatches++] = f[j];\n"
           << "        }\n"
           << "      }\n"
           << "    }\n"
           << "  }\n"
           << "  return rc;\n"
           << "}\n";
        setJavaScriptMember("GetMatchingFiles", os.str());
        
        return;
      }
      
      //----------------------------------------------------------------------
      //!  
      //----------------------------------------------------------------------
      void DropUpload::CreateDragoverListeners()
      {
        //  Remove the previous registrations
        doJavaScript(string("removeEventListener('dragenter',")
                     + jsRef() + string(".DragHover);"));
        doJavaScript(string("removeEventListener('dragleave',")
                     + jsRef() + string(".DragHover);"));

        //  Create the listener
        ostringstream  os;
        os << "function(e) {\n"
           << "  e.stopPropagation();\n"
           << "  e.preventDefault();\n"
           << "  if (e.type == 'dragenter') {\n"
           << "    if (this.EventContainsFiles(e)\n"
           << "        || this.EventContainsURIs(e)) {\n"
           << "      e.target.classList.add("
           << jsRef() << ".dragoverStyle" << ");\n"
           << "    }\n"
           << "  } else {\n"
           << "    e.target.classList.remove("
           << jsRef() << ".dragoverStyle" << ");\n"
           << "  }\n"
           << "}\n";
        setJavaScriptMember("DragHover",os.str());

        //  Reregister the listener
        doJavaScript(jsRef() + string(".addEventListener('dragenter',")
                     + jsRef()
                     + string(".DragHover,false);"));
        doJavaScript(jsRef() + string(".addEventListener('dragleave',")
                     + jsRef()
                     + string(".DragHover,false);"));
        return;
      }

      //----------------------------------------------------------------------
      //!  
      //----------------------------------------------------------------------
      void DropUpload::SetJavaScriptMimeTypes()
      {
        ostringstream  os;
        os << '[';
        auto it = _dropUploadTypes.begin();
        os << "\"" << *it << "\"";
        ++it;
        for ( ; it != _dropUploadTypes.end(); ++it) {
          os << ",\"" << *it << "\"";
        }
        os << ']';
        setJavaScriptMember("MimeTypes",os.str());
        return;
      }
        
      //----------------------------------------------------------------------
      //!  
      //----------------------------------------------------------------------
      void DropUpload::acceptDropUploads(const std::string & mimeType)
      {
        //  Convert the mimeType argument to JavaScript regular expression
        //  syntax so we can use the JavaScript string match() function
        //  to match the MIME type in our javascript without having the
        //  JavaScript do the conversion.  We expect the JavaScript to
        //  be called fairly frequently, while this function should be
        //  called rarely.
        string  mt(mimeType);
        std::regex  re("/");
        mt = regex_replace(mt, re, "\\/");
        re = "/[*]";
        mt = regex_replace(mt, re, "\\/");
        
        //  Add the mime type regular expression and hover style class to
        //  our internal set.
        _dropUploadTypes.insert(mt);
        
        //  And set our javascript array.
        SetJavaScriptMimeTypes();
        return;
      }

      //----------------------------------------------------------------------
      //!  
      //----------------------------------------------------------------------
      void DropUpload::stopAcceptDropUploads(const std::string & mimeType)
      {
        string  mt(mimeType);
        std::regex  re("/");
        mt = regex_replace(mt, re, "\\/");
        re = "/[*]";
        mt = regex_replace(mt, re, "\\/");
        _dropUploadTypes.erase(mt);
        SetJavaScriptMimeTypes();
        return;
      }

      //----------------------------------------------------------------------
      //!  
      //----------------------------------------------------------------------
      void DropUpload::setDragoverStyle(const std::string & style)
      {
        setJavaScriptMember("dragoverStyle", string("'") + style + "'");
        return;
      }

      //----------------------------------------------------------------------
      //!  
      //----------------------------------------------------------------------
      void DropUpload::setDropStyle(const std::string & style)
      {
        setJavaScriptMember("dropStyle", string("'") + style + "'");
      }
      
      //----------------------------------------------------------------------
      //!  
      //----------------------------------------------------------------------
      void DropUpload::CreateDropListener()
      {
        //  Remove the previous registration
        doJavaScript(jsRef() + string(".removeEventListener('drop',")
                     + jsRef() + string(".dropHandler);"));
        
        ostringstream  os;
        os << "function(evt) {\n"
           << "  evt.stopPropagation();\n"
           << "  evt.preventDefault();\n"
           << "  var haveFormData = false;\n"
           << "  var formData = new FormData();\n"

          //  Handle uri-list drop so I can send URL to server.  Server
          //  side can do its own fetching of the data at the URL.
           << "  var uriList = evt.dataTransfer.getData('text/uri-list');\n"
           << "  if (uriList) {\n"
           << "    formData.append('uri',uriList);\n"
           << "    haveFormData = true;\n"
           << "  }\n"
          
           << "  var matchedFiles = " << jsRef() << ".GetMatchingFiles(evt);\n"
           << "  if (matchedFiles.length > 0) {\n"
           << "    haveFormData = true;\n"
           << "    for (var f = 0; f < matchedFiles.length; ++f) {\n"
           << "      formData.append('file', matchedFiles[f]);\n"
           << "    }\n"
           << "  }\n"
           << "  if (haveFormData) {\n"
           << "    evt.target.classList.add(" << jsRef() << ".dropStyle" << ");\n"
           << "    var xhr = new XMLHttpRequest();\n"
           << "    xhr.open('POST', '" << _uploadResource.generateUrl() << "');\n"
           << "    xhr.onload = function() {\n"
           << "    evt.target.classList.remove(" << jsRef() << ".dropStyle);\n"
           << "      if (xhr.status === 200) {\n"
           << "        console.log('All done.  xhr.status ' + xhr.status);\n"
           << "      } else {\n"
           << "        console.log('Something went wrong. xhr.status ' + xhr.status);\n"
           << "      }\n"
           << "    }\n;"
           << "    xhr.send(formData);\n"
           << "  }\n"
           << "  evt.target.classList.remove("
           << jsRef() << ".dragoverStyle);\n"
           << "}\n";
        
        setJavaScriptMember("dropHandler",os.str());
        doJavaScript(jsRef() + string(".addEventListener('drop',")
                     + jsRef()
                     + string(".dropHandler,false);"));
        return;
      }
      
      
    }  // namespace Gallery

  }  // namespace WWW

}  // namespace Dwm
