//===========================================================================
// @(#) $Name:  $
// @(#) $Id: DwmWWWGalleryUploadResource.hh,v 1.5 2015/05/02 08:35:41 dwm Exp $
//===========================================================================
//  Copyright (c) Daniel W. McRobb 2015
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//  1. Redistributions of source code must retain the above copyright
//     notice, this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright
//     notice, this list of conditions and the following disclaimer in the
//     documentation and/or other materials provided with the distribution.
//  3. The names of the authors and copyright holders may not be used to
//     endorse or promote products derived from this software without
//     specific prior written permission.
//
//  IN NO EVENT SHALL DANIEL W. MCROBB BE LIABLE TO ANY PARTY FOR
//  DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
//  INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE,
//  EVEN IF DANIEL W. MCROBB HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGE.
//
//  THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND
//  DANIEL W. MCROBB HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
//  UPDATES, ENHANCEMENTS, OR MODIFICATIONS. DANIEL W. MCROBB MAKES NO
//  REPRESENTATIONS AND EXTENDS NO WARRANTIES OF ANY KIND, EITHER
//  IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//  WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE,
//  OR THAT THE USE OF THIS SOFTWARE WILL NOT INFRINGE ANY PATENT,
//  TRADEMARK OR OTHER RIGHTS.
//===========================================================================

//---------------------------------------------------------------------------
//!  \file DwmWWWGalleryUploadResource.hh
//!  \brief NOT YET DOCUMENTED
//---------------------------------------------------------------------------

#ifndef _DWMWWWGALLERYUPLOADRESOURCE_HH_
#define _DWMWWWGALLERYUPLOADRESOURCE_HH_

#include <deque>

#include <WResource>

namespace Dwm {

  namespace WWW {

    namespace Gallery {

      class DropUpload;
      
      //----------------------------------------------------------------------
      //!  This is a helper class for DropUpload.  It is not used directly
      //!  by an application.  It's main purpose is to encapsulate the
      //!  handling of an HTTP POST request containing uploaded files
      //!  and/or URIs.
      //----------------------------------------------------------------------
      class UploadResource
        : public Wt::WResource
      {
      public:
        //--------------------------------------------------------------------
        //!  Construct from a pointer to the associated DropOpload object.
        //--------------------------------------------------------------------
        UploadResource(DropUpload *dropUpload);

        //--------------------------------------------------------------------
        //!  Destructor.  Note that this will remove spool files that were
        //!  not fetched with getNextFile().
        //--------------------------------------------------------------------
        ~UploadResource();
        
        //--------------------------------------------------------------------
        //!  Handle the HTTP request containing uploaded files and/or URIs.
        //!  This is the function that will receive the POST request from
        //!  the JavaScript that uses XHR to send FormData.  See
        //!  the DropUpload implementation's JavaScript for details on
        //!  what's being sent.
        //--------------------------------------------------------------------
        void handleRequest(const Wt::Http::Request & request,
                           Wt::Http::Response & response);

        //--------------------------------------------------------------------
        //!  Gets the next uploaded file.  Returns true if there was a file
        //!  available, else returns false.  @c uploadedFile is populated
        //!  with the information about the file.
        //!  
        //!  Note that the caller is responsible for removing the uploaded
        //!  file when they are done using it.  On UNIX, this would typically
        //!  be done with unlink(uploadedFile.spoolFileName().c_str())
        //--------------------------------------------------------------------
        bool getNextFile(Wt::Http::UploadedFile & uploadedFile);
        
        //--------------------------------------------------------------------
        //!  Gets the next URI that was sent.  Returns true if a URI was
        //!  available, else returns false.  If true is returned, @c uri
        //!  will contain the URI that was sent.
        //--------------------------------------------------------------------
        bool getNextURI(std::string & uri);
        
      protected:
        DropUpload                          *_dropUpload;
        std::deque<Wt::Http::UploadedFile>   _uploadedFiles;
        std::deque<std::string>              _droppedURIs;
      };
      
    }  // namespace Gallery

  }  // namespace WWW

}  // namespace Dwm

#endif  // _DWMWWWGALLERYUPLOADRESOURCE_HH_
